@@ -119,9 +119,9 @@ struct EXTRACTOR_PluginList
    * none.
    */
 #ifndef WINDOWS
-  pid_t cpid;
+  int cpid;
 #else
-  DWORD cpid;
+  HANDLE hProcess;
 #endif
 
   /**
@@ -308,7 +308,6 @@ get_path_from_PATH() {
   char * buf;
   char * ret;
   const char * p;
-  size_t size;
 
   p = getenv("PATH");
   if (p == NULL)
@@ -322,7 +321,6 @@ get_path_from_PATH() {
       free (path);
       return NULL;
     }
-  size = strlen(path);
   pos = path;
 
   while (NULL != (end = strchr(pos, ':'))) {
@@ -851,6 +849,8 @@ EXTRACTOR_plugin_add (struct EXTRACTOR_PluginList * prev,
       return prev;
     }
   result = calloc (1, sizeof (struct EXTRACTOR_PluginList));
+  if (result == NULL)
+    return prev;
   result->next = prev;
   result->short_libname = strdup (library);
   if (result->short_libname == NULL)
@@ -957,27 +957,31 @@ stop_process (struct EXTRACTOR_PluginList *plugin)
 #endif
 
 #if DEBUG
+#ifndef WINDOWS
   if (plugin->cpid == -1)
+#else
+  if (plugin->hProcess == INVALID_HANDLE_VALUE)
+#endif
     fprintf (stderr,
 	     "Plugin `%s' choked on this input\n",
 	     plugin->short_libname);
 #endif
+#ifndef WINDOWS
   if ( (plugin->cpid == -1) ||
        (plugin->cpid == 0) )
     return;
-#ifndef WINDOWS
   kill (plugin->cpid, SIGKILL);
   waitpid (plugin->cpid, &status, 0);
-#else
-  process = OpenProcess (PROCESS_TERMINATE, FALSE, plugin->cpid);
-  TerminateProcess (process, 0);
-  CloseHandle (process);
-#endif
   plugin->cpid = -1;
-#ifndef WINDOWS
   close (plugin->cpipe_out);
   fclose (plugin->cpipe_in);
 #else
+  if (plugin->hProcess == INVALID_HANDLE_VALUE ||
+      plugin->hProcess == NULL)
+  return;
+  TerminateProcess (plugin->hProcess, 0);
+  CloseHandle (plugin->hProcess);
+  plugin->hProcess = INVALID_HANDLE_VALUE;
   CloseHandle (plugin->cpipe_out);
   CloseHandle (plugin->cpipe_in);
 #endif
@@ -1170,6 +1174,8 @@ process_requests (struct EXTRACTOR_PluginList *plugin,
 {
   char hfn[256];
   char tfn[256];
+  size_t hfn_len;
+  size_t tfn_len;
   char *fn;
   FILE *fin;
   void *ptr;
@@ -1177,6 +1183,7 @@ process_requests (struct EXTRACTOR_PluginList *plugin,
   struct IpcHeader hdr;
   size_t size;
   int want_tail;
+  int do_break;
 #ifdef WINDOWS
   HANDLE map;
 #endif
@@ -1228,17 +1235,19 @@ process_requests (struct EXTRACTOR_PluginList *plugin,
     }
   while (NULL != fgets (hfn, sizeof(hfn), fin))
     {
-      if (strlen (hfn) <= 1)
+      hfn_len = strlen (hfn);
+      if (hfn_len <= 1)
 	break;
       ptr = NULL;
-      hfn[strlen(hfn)-1] = '\0'; /* kill newline */
+      hfn[--hfn_len] = '\0'; /* kill newline */
       if (NULL == fgets (tfn, sizeof(tfn), fin))
 	break;
       if ('!' != tfn[0])
 	break;
-      tfn[strlen(tfn)-1] = '\0'; /* kill newline */
+      tfn_len = strlen (tfn);
+      tfn[--tfn_len] = '\0'; /* kill newline */
       if ( (want_tail) &&
-	   (strlen (tfn) > 1) )
+	   (tfn_len > 1) )
 	{
 	  fn = &tfn[1];
 	}
@@ -1246,7 +1255,7 @@ process_requests (struct EXTRACTOR_PluginList *plugin,
 	{
 	  fn = hfn;	
 	}
-
+      do_break = 0;
 #ifndef WINDOWS
       if ( (-1 != (shmid = shm_open (fn, O_RDONLY, 0))) &&
 	   (SIZE_MAX != (size = lseek (shmid, 0, SEEK_END))) &&
@@ -1258,15 +1267,14 @@ process_requests (struct EXTRACTOR_PluginList *plugin,
       if (ptr != NULL)
 #endif
 	{
-	  if ( (plugin->extractMethod != NULL) &&
-	       (0 != plugin->extractMethod (ptr,
-					    size,
-					    &transmit_reply,
-					    &out,
-					    plugin->plugin_options)) )
-	    break;	    
-	  if (0 != write_all (out, &hdr, sizeof(hdr)))
-	    break;
+	  if ( ( (plugin->extractMethod != NULL) &&
+		 (0 != plugin->extractMethod (ptr,
+					      size,
+					      &transmit_reply,
+					      &out,
+					      plugin->plugin_options)) ) ||
+	       (0 != write_all (out, &hdr, sizeof(hdr))) )
+	    do_break = 1;
 	}
 #ifndef WINDOWS
       if ( (ptr != NULL) &&
@@ -1280,6 +1288,8 @@ process_requests (struct EXTRACTOR_PluginList *plugin,
       if (map != NULL)
         CloseHandle (map);
 #endif
+      if (do_break)
+	break;
       if ( (plugin->specials != NULL) &&
 	   (NULL != strstr (plugin->specials,
 			    "force-kill")) )
@@ -1399,11 +1409,15 @@ start_process (struct EXTRACTOR_PluginList *plugin)
   int p2[2];
   pid_t pid;
   int status;
+#ifdef WINDOWS
+  HANDLE process;
+#endif
 
-  plugin->cpid = -1;
 #ifndef WINDOWS
+  plugin->cpid = -1;
   if (0 != pipe (p1))
 #else
+    plugin->hProcess = NULL;
     if (0 != _pipe (p1, 0, _O_BINARY))
 #endif
     {
@@ -1423,6 +1437,8 @@ start_process (struct EXTRACTOR_PluginList *plugin)
     }
 #ifndef WINDOWS
   pid = fork ();
+  plugin->cpid = pid;
+  if (pid == -1)
 #else
   STARTUPINFO startup;
   PROCESS_INFORMATION proc;
@@ -1432,13 +1448,15 @@ start_process (struct EXTRACTOR_PluginList *plugin)
   memset (&startup, 0, sizeof (STARTUPINFO));
   write_plugin_data (p1[1], plugin);
 
-
-  itoa(p1[0], arg1, 10);
-  itoa(p2[1], arg2, 10);
-  pid = _spawnl(_P_NOWAIT, "rundll32.exe", "rundll32.exe",
-        "libextractor-3.dll,RundllEntryPoint@16", arg1, arg2, NULL);
+  sprintf(cmd, "libextractor-3.dll,RundllEntryPoint@16 %u %u", p1[0], p1[1]);
+  if (CreateProcess("rundll32.exe", "libextractor-3.dll,RundllEntryPoint@16", NULL, NULL, TRUE, 0, NULL, NULL,
+      &startup, &proc))
+  {
+    plugin->hProcess = proc.hProcess;
+    CloseHandle (proc.hThread);
+  }
+  else
 #endif
-  if (pid == -1)
     {
       close (p1[0]);
       close (p1[1]);
@@ -1447,6 +1465,7 @@ start_process (struct EXTRACTOR_PluginList *plugin)
       plugin->flags = EXTRACTOR_OPTION_DISABLED;
       return;
     }
+#ifndef WINDOWS
   if (pid == 0)
     {
       close (p1[1]);
@@ -1454,18 +1473,28 @@ start_process (struct EXTRACTOR_PluginList *plugin)
       process_requests (plugin, p1[0], p2[1]);
       _exit (0);
     }
-  plugin->cpid = pid;
+#endif
   close (p1[0]);
   close (p2[1]);
   plugin->cpipe_in = fdopen (p1[1], "w");
   if (plugin->cpipe_in == NULL)
     {
       perror ("fdopen");
+#ifndef WINDOWS
       (void) kill (plugin->cpid, SIGKILL);
       waitpid (plugin->cpid, &status, 0);
+#else
+      TerminateProcess (plugin->hProcess, 0);
+      WaitForSingleObject (process, INFINITE);
+      CloseHandle (plugin->hProcess);
+#endif
       close (p1[1]);
       close (p2[0]);
+#ifndef WINDOWS
       plugin->cpid = -1;
+#else
+      plugin->hProcess = INVALID_HANDLE_VALUE;
+#endif
       plugin->flags = EXTRACTOR_OPTION_DISABLED;
       return;
     }
@@ -1495,14 +1524,22 @@ extract_oop (struct EXTRACTOR_PluginList *plugin,
   char mimetype[MAX_MIME_LEN + 1];
   char *data;
 
+#ifndef WINDOWS
   if (plugin->cpid == -1)
+#else
+  if (plugin->hProcess == INVALID_HANDLE_VALUE)
+#endif
     return 0;
   if (0 >= fprintf (plugin->cpipe_in, 
 		    "%s\n",
 		    shmfn))
     {
       stop_process (plugin);
+#ifndef WINDOWS
       plugin->cpid = -1;
+#else
+      plugin->hProcess = INVALID_HANDLE_VALUE;
+#endif
       if (plugin->flags != EXTRACTOR_OPTION_DEFAULT_POLICY)
 	plugin->flags = EXTRACTOR_OPTION_DISABLED;
       return 0;
@@ -1512,7 +1549,11 @@ extract_oop (struct EXTRACTOR_PluginList *plugin,
 		    (tshmfn != NULL) ? tshmfn : ""))
     {
       stop_process (plugin);
+#ifndef WINDOWS
       plugin->cpid = -1;
+#else
+      plugin->hProcess = INVALID_HANDLE_VALUE;
+#endif
       if (plugin->flags != EXTRACTOR_OPTION_DEFAULT_POLICY)
 	plugin->flags = EXTRACTOR_OPTION_DISABLED;
       return 0;
@@ -1525,7 +1566,11 @@ extract_oop (struct EXTRACTOR_PluginList *plugin,
 			 sizeof(hdr)))
 	{
 	  stop_process (plugin);
-	  plugin->cpid = -1;
+#ifndef WINDOWS
+      plugin->cpid = -1;
+#else
+      plugin->hProcess = INVALID_HANDLE_VALUE;
+#endif
 	  if (plugin->flags != EXTRACTOR_OPTION_DEFAULT_POLICY)
 	    plugin->flags = EXTRACTOR_OPTION_DISABLED;
 	  return 0;
@@ -1538,7 +1583,11 @@ extract_oop (struct EXTRACTOR_PluginList *plugin,
       if (hdr.mime_len > MAX_MIME_LEN)
 	{
 	  stop_process (plugin);	  
-	  plugin->cpid = -1;
+#ifndef WINDOWS
+      plugin->cpid = -1;
+#else
+      plugin->hProcess = INVALID_HANDLE_VALUE;
+#endif
 	  if (plugin->flags != EXTRACTOR_OPTION_DEFAULT_POLICY)
 	    plugin->flags = EXTRACTOR_OPTION_DISABLED;
 	  return 0;
@@ -1557,7 +1606,11 @@ extract_oop (struct EXTRACTOR_PluginList *plugin,
 			    hdr.data_len))) )
 	{
 	  stop_process (plugin);
-	  plugin->cpid = -1;
+#ifndef WINDOWS
+      plugin->cpid = -1;
+#else
+      plugin->hProcess = INVALID_HANDLE_VALUE;
+#endif
 	  free (data);
 	  if (plugin->flags != EXTRACTOR_OPTION_DEFAULT_POLICY)
 	    plugin->flags = EXTRACTOR_OPTION_DISABLED;
@@ -1632,8 +1685,9 @@ make_shm (int is_tail,
        (NULL == (*ptr = mmap (NULL, size, PROT_WRITE, MAP_SHARED, *shmid, 0))) ||
        (*ptr == (void*) -1) )
     {
-      close (*shmid);	
+      close (*shmid);
       *shmid = -1;
+      shm_unlink (fn);
       return 1;
     }
   return 0;
@@ -1700,13 +1754,21 @@ extract (struct EXTRACTOR_PluginList *plugins,
       switch (ppos->flags)
 	{
 	case EXTRACTOR_OPTION_DEFAULT_POLICY:
+#ifndef WINDOWS
 	  if ( (0 == ppos->cpid) ||
 	       (-1 == ppos->cpid) )
+#else
+	  if (ppos->hProcess == NULL || ppos->hProcess == INVALID_HANDLE_VALUE)
+#endif
 	    start_process (ppos);
 	  want_shm = 1;
 	  break;
 	case EXTRACTOR_OPTION_OUT_OF_PROCESS_NO_RESTART:
+#ifndef WINDOWS
 	  if (0 == ppos->cpid)
+#else
+	  if (ppos->hProcess == NULL)
+#endif
 	    start_process (ppos);
 	  want_shm = 1;
 	  break;
@@ -1769,21 +1831,34 @@ extract (struct EXTRACTOR_PluginList *plugins,
 	  if (0 != extract_oop (ppos, fn, 
 				(tptr != NULL) ? tfn : NULL,
 				proc, proc_cls))
-	    return;
+	    {
+	      ppos = NULL;
+	      break;
+	    }
+#ifndef WINDOWS
 	  if (ppos->cpid == -1)
+#else
+      if (ppos->hProcess == INVALID_HANDLE_VALUE)
+#endif
 	    {
 	      start_process (ppos);
 	      if (0 != extract_oop (ppos, fn, 
 				    (tptr != NULL) ? tfn : NULL,
 				    proc, proc_cls))
-		return;
+		{
+		  ppos = NULL;
+		  break;
+		}
 	    }
 	  break;
 	case EXTRACTOR_OPTION_OUT_OF_PROCESS_NO_RESTART:
 	  if (0 != extract_oop (ppos, fn,
 				(tptr != NULL) ? tfn : NULL,
 				proc, proc_cls))
-	    return;
+	    {
+	      ppos = NULL;
+	      break;
+	    }
 	  break;
 	case EXTRACTOR_OPTION_IN_PROCESS:	  	  
 	  want_tail = ( (ppos->specials != NULL) &&
@@ -1804,7 +1879,10 @@ extract (struct EXTRACTOR_PluginList *plugins,
 						  proc, 
 						  proc_cls,
 						  ppos->plugin_options)) )
-		    return;
+		    {
+		      ppos = NULL;
+		      break;
+		    }
 		}
 	      else
 		{
@@ -1814,13 +1892,18 @@ extract (struct EXTRACTOR_PluginList *plugins,
 						  proc, 
 						  proc_cls,
 						  ppos->plugin_options)) )
-		    return;
+		    {
+		      ppos = NULL;
+		      break;
+		    }
 		}
 	    }
 	  break;
 	case EXTRACTOR_OPTION_DISABLED:
 	  break;
 	}      
+      if (ppos == NULL)
+	break;
       ppos = ppos->next;
     }
   if (want_shm)
@@ -2240,12 +2323,7 @@ EXTRACTOR_extract (struct EXTRACTOR_PluginList *plugins,
        (fstatbuf.st_size > fsize) &&
        (fstatbuf.st_size > MAX_READ) )
     {
-#ifndef WINDOWS
-      pg = sysconf (_SC_PAGE_SIZE);
-#else
-      GetSystemInfo(&sys);
-      pg = sys.dwPageSize;
-#endif
+      pg = SYSCONF (_SC_PAGE_SIZE);
       if ( (pg > 0) &&
 	   (pg < MAX_READ) )
 	{
