@@ -299,7 +299,7 @@ static const struct MetaTypeDescription meta_type_descriptions[] = {
   { gettext_noop ("line count"),
     gettext_noop ("number of lines") }, 
   { gettext_noop ("paragraph count"),
-    gettext_noop ("number o paragraphs") }, 
+    gettext_noop ("number of paragraphs") }, 
   /* 120 */
   { gettext_noop ("word count"),
     gettext_noop ("number of words") }, 
@@ -433,6 +433,8 @@ static const struct MetaTypeDescription meta_type_descriptions[] = {
   { gettext_noop ("group"), 
     gettext_noop ("name of the group or band") },
   /* 180 */
+  { gettext_noop ("original filename"),
+    gettext_noop ("name of the original file (reserved for GNUnet)") },
   { gettext_noop ("last"),
     gettext_noop ("last") }
 };
