@@ -46,7 +46,6 @@ parseZZZ (const char *data,
   size_t slen;
   size_t end;
   int i;
-  char *value;
 
   end = pos + len;
   slen = strlen ("ps:SDict begin [");
@@ -70,24 +69,21 @@ parseZZZ (const char *data,
                   while ((slen < end) && (data[slen] != ')'))
                     slen++;
                   slen = slen - pos;
-                  value = malloc (slen + 1);
-		  if (value != NULL)
-		    {
-		      value[slen] = '\0';
-		      memcpy (value, &data[pos], slen);
-		      if (0 != proc (proc_cls, 
-				     "dvi",
-				     tmap[i].type,
-				     EXTRACTOR_METAFORMAT_C_STRING,
-				     "text/plain",
-				     value,
-				     strlen (value) +1))
-			{
-			  free (value);
-			  return 1;
-			}
-		      free (value);
-		    }
+		  {
+		    char value[slen + 1];
+		    value[slen] = '\0';
+		    memcpy (value, &data[pos], slen);
+		    if (0 != proc (proc_cls, 
+				   "dvi",
+				   tmap[i].type,
+				   EXTRACTOR_METAFORMAT_C_STRING,
+				   "text/plain",
+				   value,
+				   slen +1))
+		      {
+			return 1;
+		      }
+		  }
                   pos += slen + 1;
                 }
             }
@@ -125,7 +121,6 @@ EXTRACTOR_dvi_extract (const unsigned char *data,
 		       const char *options)
 {
   unsigned int klen;
-  char *comment;
   uint32_t pos;
   uint32_t opos;
   unsigned int len;
@@ -192,24 +187,20 @@ EXTRACTOR_dvi_extract (const unsigned char *data,
 		 "application/x-dvi",
 		 strlen ("application/x-dvi") +1))
     return 1;
-  comment = malloc (klen + 1);
-  if (comment != NULL)
-    {
-      comment[klen] = '\0';
-      memcpy (comment, &data[15], klen);
-      if (0 != proc (proc_cls, 
-		     "dvi",
-		     EXTRACTOR_METATYPE_COMMENT,
-		     EXTRACTOR_METAFORMAT_UTF8,
-		     "text/plain",
-		     comment,
-		     strlen (comment) +1))
-	{
-	  free (comment);
-	  return 1;
-	}
-      free (comment);
-    }
+  {
+    char comment[klen + 1];
+    
+    comment[klen] = '\0';
+    memcpy (comment, &data[15], klen);
+    if (0 != proc (proc_cls, 
+		   "dvi",
+		   EXTRACTOR_METATYPE_COMMENT,
+		   EXTRACTOR_METAFORMAT_UTF8,
+		   "text/plain",
+		   comment,
+		   klen +1))
+      return 1;
+  }
   /* try to find PDF/ps special */
   pos = opos;
   while (pos < size - 100)
