@@ -68,41 +68,49 @@ libextractor_oo_getmimetype(EXTRACTOR_unzip_file uf) {
   char * buf = NULL;
   size_t buf_size = 0;
 
-  if (EXTRACTOR_UNZIP_OK == EXTRACTOR_common_unzip_local_file(uf,
-			      "mimetype",
-			      CASESENSITIVITY)) {
-    if ( (EXTRACTOR_UNZIP_OK == EXTRACTOR_common_unzip_get_current_file_info(uf,
-					  &file_info,
-					  filename_inzip,
-					  sizeof(filename_inzip),
-					  NULL,
-					  0,
-					  NULL,
-					  0) &&
-	  (EXTRACTOR_UNZIP_OK == EXTRACTOR_common_unzip_open_current_file3(uf,NULL, NULL, 0)) ) ) {
-      buf_size = file_info.uncompressed_size;
-
-      if (buf_size > 1024) {
+  if (EXTRACTOR_UNZIP_OK != EXTRACTOR_common_unzip_local_file(uf,
+							      "mimetype",
+							      CASESENSITIVITY))
+    return NULL;
+  if ( (EXTRACTOR_UNZIP_OK == EXTRACTOR_common_unzip_get_current_file_info(uf,
+									   &file_info,
+									   filename_inzip,
+									   sizeof(filename_inzip),
+									   NULL,
+									   0,
+									   NULL,
+									   0) &&
+	(EXTRACTOR_UNZIP_OK == EXTRACTOR_common_unzip_open_current_file3(uf, NULL, NULL, 0)) ) ) {
+    buf_size = file_info.uncompressed_size;
+    
+    if (buf_size > 1024) 
+      {
 	/* way too large! */
-      } else if (NULL == (buf = malloc(1 + buf_size))) {
-	/* memory exhausted! */
-      } else if (buf_size != (size_t) EXTRACTOR_common_unzip_read_current_file(uf,buf,buf_size)) {
-        free(buf);
-        buf = NULL;
-      } else {
-	/* found something */
-        buf[buf_size] = '\0';
-        while ( (0 > buf_size) &&
-		isspace( (unsigned char) buf[buf_size - 1]))
-          buf[--buf_size] = '\0';
-        if('\0' == buf[0]) {
-          free(buf);
-          buf = NULL;
-        }
       }
-    }
-    EXTRACTOR_common_unzip_close_current_file(uf);
+    else if (NULL == (buf = malloc(1 + buf_size))) 
+      {
+	/* memory exhausted! */
+      } 
+    else if (buf_size != (size_t) EXTRACTOR_common_unzip_read_current_file(uf,buf,buf_size)) 
+      {
+	free(buf);
+	buf = NULL;
+      }
+    else 
+      {
+	/* found something */
+	buf[buf_size] = '\0';
+	while ( (0 < buf_size) &&
+		isspace( (unsigned char) buf[buf_size - 1]))
+	  buf[--buf_size] = '\0';
+	if ('\0' == buf[0]) 
+	  {
+	    free(buf);
+	    buf = NULL;
+	  }
+      }
   }
+  EXTRACTOR_common_unzip_close_current_file(uf);  
   return buf;
 }
 
@@ -196,18 +204,20 @@ EXTRACTOR_odf_extract (const char *data,
     return 0; /* hardly meta-data! */
   }
   buf = malloc(buf_size+1);
-  if (buf == NULL) {
-    EXTRACTOR_common_unzip_close_current_file(uf);
-    EXTRACTOR_common_unzip_close(uf);
-    return 0; /* out of memory */
-  }
+  if (buf == NULL) 
+    {
+      EXTRACTOR_common_unzip_close_current_file(uf);
+      EXTRACTOR_common_unzip_close(uf);
+      return 0; /* out of memory */
+    }
 
-  if (buf_size != EXTRACTOR_common_unzip_read_current_file(uf,buf,buf_size)) {
-    free(buf);
-    EXTRACTOR_common_unzip_close_current_file(uf);
-    EXTRACTOR_common_unzip_close(uf);
-    return 0;
-  }
+  if (buf_size != EXTRACTOR_common_unzip_read_current_file(uf,buf,buf_size)) 
+    {
+      free(buf);
+      EXTRACTOR_common_unzip_close_current_file(uf);
+      EXTRACTOR_common_unzip_close(uf);
+      return 0;
+    }
   EXTRACTOR_common_unzip_close_current_file(uf);
   /* we don't do "proper" parsing of the meta-data but rather use some heuristics
      to get values out that we understand */
@@ -223,7 +233,6 @@ EXTRACTOR_odf_extract (const char *data,
       char * spos;
       char * epos;
       char needle[256];
-      char * key;
       int oc;
 
       pbuf = buf;
@@ -263,24 +272,27 @@ EXTRACTOR_odf_extract (const char *data,
 	    epos++;
 	  }
 	}
-	if (spos != epos) {
-	  key = malloc(1+epos-spos);
-	  memcpy(key, spos, epos-spos);
-	  key[epos-spos] = '\0';
-	  if (0 != proc (proc_cls, 
-			 "odf",
-			 tmap[i].type,
-			 EXTRACTOR_METAFORMAT_UTF8,
-			 "text/plain",
-			 key,
-			 strlen (key)+1))
-	    {
-	      free(buf);
-	      EXTRACTOR_common_unzip_close(uf);
-	      return 1;	      
-	    }
-	  pbuf = epos;
-	} else
+	if (spos != epos) 
+	  {
+	    char key[epos - spos + 1];
+
+	    memcpy(key, spos, epos-spos);
+	    key[epos-spos] = '\0';
+	    if (0 != proc (proc_cls, 
+			   "odf",
+			   tmap[i].type,
+			   EXTRACTOR_METAFORMAT_UTF8,
+			   "text/plain",
+			   key,
+			   epos - spos + 1))
+	      {
+		free(buf);
+		EXTRACTOR_common_unzip_close(uf);
+		return 1;	      
+	      }	  
+	    pbuf = epos;
+	  } 
+	else
 	  break;
       }
     }
@@ -290,3 +302,4 @@ EXTRACTOR_odf_extract (const char *data,
   return 0;
 }
 
+
